#ifndef __DEBUG_exec
#undef  DEBUG
#endif

#define __module_name__ 	"exec"

#include "types.h"
#include "param.h"
#include "memlayout.h"
#include "hal/riscv.h"
#include "sched/proc.h"
#include "elf.h"
#include "fs/fs.h"
#include "mm/pm.h"
#include "mm/vm.h"
#include "mm/usrmm.h"
#include "printf.h"
#include "utils/string.h"
#include "syscall.h"
#include "utils/debug.h"
#include "errno.h"
#include "mm/mmap.h"
// #include "elf.h"

// Load a program segment into pagetable at virtual address va.
// va must be page-aligned (not necessary)
// and the pages from va to va+sz must already be mapped.
// Returns 0 on success, -1 on failure.
/*
static int
loadseg(pagetable_t pagetable, uint64 va, struct inode *ip, uint offset, uint sz)
{
	uint i, n;
	uint64 pa;
	// if((va % PGSIZE) != 0)
	//   panic("loadseg: va must be page aligned");

	for (i = 0; i < sz; i += n) {
		pa = walkaddr(pagetable, va + i);
		if (pa == NULL)
			panic("loadseg: address should exist");

		uint m = (va + i) % PGSIZE; 
		if (sz - i < PGSIZE)
			n = sz - i;
		else
			n = PGSIZE - m;

		// __debug_info("loadseg", "pa=%p, n=%d\n", pa + m, n);
		if(ip->fop->read(ip, 0, pa + m, offset + i, n) != n)
			return -1;
	}

	return 0;
}
*/

// push arg or env strings into user stack, return strings count
static int pushstack(pagetable_t pt, uint64 table[], char **utable, int maxc, uint64 *spp)
{
	uint64 sp = *spp;
	uint64 stackbase = PGROUNDDOWN(sp);
	uint64 argc, argp;
	char *buf;

	if ((buf = allocpage()) == NULL) {
		return -ENOMEM;
	}
	for (argc = 0; utable; argc++) {
		if (argc >= maxc || fetchaddr((uint64)(utable + argc), &argp) < 0)
			goto bad;
		__debug_info("pushstack", "argp=%p\n", argp);
		if (argp == 0)
			break;
		// IMP: in fetchstr using copyinstr2
		int arglen = fetchstr(argp, buf, PGSIZE);   // '\0' included in PGSIZE, but not in envlen
		__debug_info("pushstack", "arglen = %d\n", arglen);
		if (arglen++ < 0) {                               // including '\0'
			__debug_warn("pushstack", "didn't get null\n");
			goto bad;
		}
		sp -= arglen;
		if (sp < stackbase || copyout(pt, sp, buf, arglen) < 0)
			goto bad;
		table[argc] = sp;
	}
	table[argc] = 0;
	*spp = sp;
	freepage(buf);
	return argc;

bad:
	if (buf)
		freepage(buf);
	return -EFAULT;
}

#define min(x, y) ((x)<(y)? (x):(y))
#define max(x, y) ((x)>(y)? (x):(y))

uint64
size4mapping(struct elfhdr* interp_elfhdr, struct inode* interpreter){
  uint64 low_bound = -1, upper_bound = 0;
  int valid = 0;
  struct proghdr ph;
  // 先理解好interpreter这个文件的意义
  for(uint i = 0, off = interp_elfhdr->phoff; i < interp_elfhdr->phnum; i++, off += sizeof(struct proghdr)){
    if(interpreter->fop->read(interpreter, 0, (uint64)&ph, off, sizeof(struct proghdr)) != sizeof(struct proghdr)){
      printf("[get_total_mapping_size]eread failed\n");
      return -1;
    }
    if(ph.type == ELF_PROG_LOAD){
      low_bound = min(low_bound, PGROUNDDOWN(ph.vaddr)); //读取虚拟地址大小
      upper_bound = max(upper_bound, ph.vaddr + ph.memsz);
      valid = 1;
    }
  }
  return valid ? (upper_bound - low_bound) : 0;
}


uint64
load_elf_interp(pagetable_t pagetable, struct seg* seghead, struct elfhdr* interp_elfhdr, struct inode* interpreter){
  uint64 whole = 0;
  uint64 start_addr = 0;
  struct proghdr ph;
  struct seg *seg,*newseg;
  // 计算需要装载的总大小
  whole = size4mapping(interp_elfhdr, interpreter);
  if(whole == 0){
    printf("[load_elf_interp]get_total_mapping_size failed\n");
    return -1;
  }
  // 使用mmap来完成对这一部分的装载
  // start_addr = mmap(0, whole, PROT_READ | PROT_WRITE | PROT_EXEC, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
  // consider the permission, and the constants defined.
  // 注意映射的类型
  start_addr = do_mmap2seg(0,whole,PROT_READ | PROT_WRITE | PROT_EXEC, MAP_ANONYMOUS | MAP_PRIVATE, 0, 0,&newseg);
  // start_addr = do_mmap(0,whole,PROT_READ | PROT_WRITE | PROT_EXEC, MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
  if(start_addr == -1){
    printf("[load_elf_interp]do_mmap failed\n");
    return -1;
  }
  // 开始加载
  for(uint i = 0, off = interp_elfhdr->phoff; i < interp_elfhdr->phnum; i++, off += sizeof(struct proghdr)){
    if(interpreter->fop->read(interpreter, 0, (uint64)&ph, off, sizeof(struct proghdr)) != sizeof(struct proghdr)){
      printf("[load_elf_interp]eread failed\n");
      return -1;
    }
    if(ph.type == ELF_PROG_LOAD){
      if(ph.memsz < ph.filesz){
        printf("[load_elf_interp]ph.memsz < ph.filesz\n");
        return -1;
      }
      if(ph.vaddr + ph.memsz < ph.vaddr){
        printf("[load_elf_interp]ph.vaddr + ph.memsz < ph.vaddr\n");
        return -1;
      }
	  seg = locateseg(seghead, ph.vaddr);
	  seg->f_off = ph.off;
	  seg->f_sz = ph.filesz;
      if(loadseg(pagetable, start_addr + ph.vaddr, seg, interpreter) < 0){
        // 转交给子函数load
        printf("[load_elf_interp]loadseg failed\n");
        return -1;
      }
    }
  }
  return start_addr;
}

// All argvs are pointers came from user space, and should be checked by sys_caller
int execve(char *path, char **argv, char **envp)
{
	__debug_info("exeve", "path = %s\n", path);
	int ret;
	struct inode *ip = NULL;
	pagetable_t pagetable = NULL;
	struct seg *seghead = NULL, *seg;
	struct proc *p = myproc();
	// 
	int dynamic_need = 0;
	struct proghdr ph_store;
	ph_store.vaddr = 0; // to avoid warning
	int already_store = 0;

	__debug_info("execve", "in\n");
	if ((ip = namei(path)) == NULL) {
		__debug_warn("execve", "can't open %s\n", path);
		ret = -ENOENT;
		goto bad;
	}

	// Make a copy of p->pagetable without old user space, 
	// but with the same kstack we are using now, which can't be changed.
	if ((pagetable = (pagetable_t)allocpage()) == NULL) {
		ret = -ENOMEM;
		goto bad;
	}
	memmove(pagetable, p->pagetable, PGSIZE); // understand this and below
	// Remove old u-map from the new pt, but don't free since later op might fail.
	for (int i = 0; i < PX(2, MAXUVA); i++) {
		pagetable[i] = 0;
	}

	__debug_info("execve", "get elf\n");
	// Check ELF header
	//printf("NNN: exe lock elf_ip\n");
	ilock(ip);
	struct elfhdr elf;
	if (ip->fop->read(ip, 0, (uint64) &elf, 0, sizeof(elf)) != sizeof(elf) || elf.magic != ELF_MAGIC) { // check if the bytes read were right
		iunlock(ip);
		ret = -ENOEXEC;
		goto bad;
	}

	//printf("NNN:IN exe_p1 %ip->ref = %d\n",ip->ref);

	__debug_info("execve", "load from elf\n");
	// Load program (elf file) into memory. ----could be packed in a function
	struct proghdr ph;
	// the usage of elfaddr was replaced by ph_store
	// uint64 elfaddr = 0;
	int flags;
	for (int i = 0, off = elf.phoff; i < elf.phnum; i++, off += sizeof(ph)) {
		if (ip->fop->read(ip, 0, (uint64)&ph, off, sizeof(ph)) != sizeof(ph)) {
			__debug_warn("execve", "fail to read ELF file\n");
			iunlock(ip);
			ret = -EIO;
			goto bad;
		}
		if (ph.type == ELF_PROG_LOAD){ // 写错条件了, 导致直接没有load, 从而触发了各种访存异常
			// goto normal1; -------- now we reorganize the layout of the code, thus solved
			// if (ph.vaddr % PGSIZE != 0 || ph.memsz < ph.filesz || ph.vaddr + ph.memsz < ph.vaddr) {
			if (ph.memsz < ph.filesz || ph.vaddr + ph.memsz < ph.vaddr) {
				__debug_warn("execve", "funny ELF file: va=%p msz=0x%x fsz=0x%x\n", ph.vaddr, ph.memsz, ph.filesz);
					iunlock(ip);
				ret = -ENOEXEC;
				goto bad;
			}

			// turn ELF flags to PTE flags
			flags = 0;
			flags |= (ph.flags & ELF_PROG_FLAG_EXEC) ? PTE_X : 0;
			flags |= (ph.flags & ELF_PROG_FLAG_WRITE) ? PTE_W : 0;
			flags |= (ph.flags & ELF_PROG_FLAG_READ) ? PTE_R : 0;
			seg = newseg(pagetable, seghead, LOAD, ph.vaddr, ph.memsz, flags);
			if(seg == NULL) {
				__debug_warn("execve", "newseg fail: vaddr=%p, memsz=%d\n", ph.vaddr, ph.memsz);
				iunlock(ip);
				ret = -ENOMEM;
				goto bad;
			}
			seghead = seg;

			if (ph.off == 0){
				// the usage of elfaddr was replaced by ph_store
				// elfaddr = ph.vaddr;
				if(!already_store){
					// see the documents to know the meaning of the field in the struct
					ph_store.vaddr = elf.phoff + ph.vaddr;
				}
			}
			

			seg = locateseg(seghead, ph.vaddr);
			seg->f_off = ph.off;
			seg->f_sz = ph.filesz;
			if (ph.vaddr <= elf.entry && elf.entry < ph.vaddr + ph.filesz) {
				__debug_info("execve", "meet entry segment: entry=0x%x, va=0x%x, sz=0x%x\n", elf.entry, ph.vaddr, ph.filesz);
				if ((ret = loadseg(pagetable, elf.entry, seg, ip)) < 0) {
					__debug_warn("execve", "load segment\n");
					iunlock(ip);
					goto bad;
				}
			}
		}
		else if(ph.type == ELF_PROG_PHDR){
			ph_store = ph;
			already_store = 1;
		}else if(ph.type == ELF_PROG_INTERP){
			dynamic_need = 1;
		}else{
			//printf("ph.type: %d ph.type != ELF_PROG_LOAD\n", ph.type);
		}
// normal1:
	}

	// NOTE!!!  since MMAP is in the higher address than HEAP, we should firstly alloc HEAP, and then using mmap to do dynamic link
	// Heap
	flags = PTE_R | PTE_W;
	for (seg = seghead; seg->next != NULL; seg = seg->next); // reach the end of the list
	uint64 brk = PGROUNDUP(seg->addr + seg->sz) + PGSIZE;
	__debug_info("execve", "making heap, start=%p\n", brk);
	seg = newseg(pagetable, seghead, HEAP, brk, 0, flags);
	if(seg == NULL) {
		__debug_warn("execve", "new heap fail\n");
		ret = -ENOMEM;
		goto bad;
	}
	seghead = seg;

	// when dynamic linking is needed
	struct inode *intprtr;
	struct elfhdr intprtr_hdr;

	uint64 prog_entry, intprtr_sta_loc;

	if(dynamic_need){
		if((intprtr = namei("/libc.so")) == NULL){
			printf("interpreter not found\n");
			goto bad;
		}
		//printf("NNN: exe lock interpr_ip\n");
		ilock(intprtr);
		if(intprtr->fop->read(intprtr, 0, (uint64) &elf, 0, sizeof(intprtr_hdr)) != sizeof(intprtr_hdr) ){
			printf("read interpreter_hdr failed\n");
      		goto bad;
		}
		// todo:
		intprtr_sta_loc = load_elf_interp(pagetable,seghead, &intprtr_hdr, intprtr);
		// 看一下计算机系统基础上的动态链接的原理，思考相对定位和内存映射的作用
		prog_entry = intprtr_sta_loc + intprtr_hdr.entry;
		//printf("interp_start_addr:%p program_entry:%p\n", interp_start_addr, program_entry);

		iunlockput(intprtr);
		intprtr = NULL;
	}
	else{
		prog_entry = elf.entry;
		intprtr_sta_loc = 0;
	}
	// Ques
	// iunlockput(ip);
	// //printf("NNN:IN exe_p1 %ip->ref = %d\n",ip->ref);
	iunlock(ip); // IMP: when to put?
	// ip = NULL; // to tell bad about whether is has already been put and unlocked
	//printf("NNN:IN exe_p1.5 %ip->ref = %d\n",ip->ref);

	__debug_info("execve", "making stack\n");
	// Stack
	uint64 sp = VUSTACK;
	#define STACK_PAGES	35
	// STACK位于MMAP的下方,与之错开, 两者以0x7000_0000为分界线
	uint64 stackbase = VUSTACK - PGSIZE * STACK_PAGES;
	seg = newseg(pagetable, seghead, STACK, stackbase, sp - stackbase, flags);
	stackbase += PGSIZE * (STACK_PAGES - 1);
	if(seg == NULL || uvmalloc(pagetable, stackbase, sp, flags) == 0) {
		__debug_warn("execve", "new stack fail\n");
		ret = -ENOMEM;
		goto bad;
	}
	seghead = seg;

	// prepare a NULL on the bottom of stack
	sp -= sizeof(uint64);
	__debug_info("execve", "pushing argv/envp\n");
	// arguments to user main(argc, argv, envp)
	// argc is returned via the system call return
	// value, which goes in a0.
	int64 argc, envc;
	uint64 uargv[MAXARG + 1], uenvp[MAXENV + 1];
	if ((envc = pushstack(pagetable, uenvp, envp, MAXENV, &sp)) < 0 ||
		(argc = pushstack(pagetable, uargv, argv, MAXARG, &sp)) < 0) {
		__debug_warn("execve", "fail to push argv or envp into user stack\n");
		ret = envc < 0 ? envc : argc;
		goto bad;
	}
	uint64 random[2] = { 0xcde142a16cb93072, 0x128a39c127d8bbf2 };
	sp -= 16;
	if (sp < stackbase || copyout(pagetable, sp, (char *)random, 16) < 0) {
		ret = -EINVAL;
		printf("In execve, write random copy bad\n");
		goto bad;
	}

	uint64 auxvec[][2] = {
		// {AT_HWCAP, 0x112d},
		{AT_HWCAP, 0},
		{AT_PAGESZ, PGSIZE},
		{AT_PHDR, ph_store.vaddr},
		{AT_PHENT, elf.phentsize},
		{AT_PHNUM, elf.phnum},
		{AT_BASE, intprtr_sta_loc}, // 在这里传递动态链接的信息
		{AT_ENTRY, elf.entry},
		{AT_UID, 0},
		{AT_EUID, 0},
		{AT_GID, 0},
		{AT_EGID, 0},
		{AT_SECURE, 0},
		{AT_RANDOM, sp},
		{AT_NULL, 0}
	};

	sp -= 16 + sizeof(auxvec);
	sp -= (envc + 1) * sizeof(uint64);
	uint64 a2 = sp;
	sp -= (argc + 1) * sizeof(uint64);
	uint64 a1 = sp;
	sp -= sizeof(uint64); // argc
	flags = sp % 16;      // on risc-v, sp should be 16-byte aligned
	sp -= flags;
	a1 -= flags;
	a2 -= flags;

	__debug_info("execve", "pushing argv/envp table\n");
	if (sp < stackbase || 
		copyout(pagetable, a2 + (envc + 1) * sizeof(uint64), (char *)auxvec, sizeof(auxvec)) < 0 ||
		copyout(pagetable, a2, (char *)uenvp, (envc + 1) * sizeof(uint64)) < 0 ||
		copyout(pagetable, a1, (char *)uargv, (argc + 1) * sizeof(uint64)) < 0 ||
		copyout(pagetable, sp, (char *)&argc, sizeof(uint64)) < 0)
	{
		__debug_warn("execve", "fail to copy argv/envp table into user stack\n");
		ret = -EINVAL;
		goto bad;
	}
	p->trapframe->a1 = a1;
	p->trapframe->a2 = a2;

	// Save program name for debugging.
	//printf("NNN:IN exe_p1.6 %ip->ref = %d\n",ip->ref);
	safestrcpy(p->name, ip->entry->filename, sizeof(p->name));
	//printf("NNN:IN exe_p1.7 %ip->ref = %d\n",ip->ref);

	// Commit to the user image.
	pagetable_t oldpagetable = p->pagetable;
	seg = p->segment;
	p->pagetable = pagetable;
	p->segment = seghead;
	p->pbrk = brk;
	p->trapframe->epc = prog_entry;  // initial program counter = main
	p->trapframe->sp = sp; // initial stack pointer
	//printf("NNN:IN exe_p1.8 %ip->ref = %d\n",ip->ref);
	// if (p->elf)
	if (p->elf&&ip!=p->elf)
		iput(p->elf);
	p->elf = ip;
	//printf("NNN:IN exe_p2 %ip->ref = %d\n",ip->ref);
	// p->elf = idup(ip);

	__debug_info("execve", "sp=%p, stackbase=%p, entry=%p\n", sp, stackbase, elf.entry);
	fdcloexec(&p->fds);
	w_satp(MAKE_SATP(p->pagetable));
	sfence_vma();
	w_sstatus_fs(SSTATUS_FS_CLEAN);

	delsegs(oldpagetable, seg);
	uvmfree(oldpagetable);
	freepage(oldpagetable);

	return 0;
	// return argc; // this ends up in a0, the first argument to main(argc, argv)

	bad:
	__debug_warn("execve", "reach bad: seg=%p, pt=%p, ip=%p\n", seghead, pagetable, ip);
	if (seghead) {
		delsegs(pagetable, seghead);
	}
	if (pagetable) {
		uvmfree(pagetable);
		freepage(pagetable);
	}
	if (ip) {
		iput(ip);
	}
	return ret;
}